import re
from wiktionaryparser import WiktionaryParser
import json
from tqdm import tqdm
import os.path

from clippings2anki.flashcards import get_flashcard
from clippings2anki.anki import save_to_text
import clippings2anki.gui


def main_gui():
    clippings2anki.gui.main()


def main_cli(input_file, language, output_file, save_json=False, anki_separator="\t", qt_signal=None):
    #! read the clippings file
    with open(input_file, "r") as f:
        clippings = f.read()
    
    clip_re = re.compile(r"(.*)\n==========")
    
    # find all matches
    clips = clip_re.findall(clippings)
    
    clips_set = set()
    
    for clip in clips:
        if not clip:
            continue

        # remove punctuation from the clip
        clip = re.sub(r"[^\w\s]", "", clip)

        # print(clip)
        clips_set.add(clip)
        
    print(f"{len(clips_set)} clippings found")
    print()

    #! get the definitions of the words from wiktionary
    parser = WiktionaryParser()
    parser.set_default_language(language)
    
    flashcards = {}

    for i, clip in enumerate(tqdm(clips_set, desc="Getting definitions from wiktionary")):
        word, f = get_flashcard(clip, parser)
        
        if not word:
            continue
        
        flashcards[word] = f
        
        if qt_signal:
            qt_signal.emit(int(((i+1) / len(clips_set))*100))
    
    if qt_signal:
        qt_signal.emit(100)
    
    print(f"\n{len(flashcards)} flashcards created out of {len(clips_set)} clippings ({len(clips_set) - len(flashcards)} clippings discarded)")
    
    if save_json:
        # save the flashcards to json file
        json_file = os.path.splitext(output_file)[0] + ".json"
        with open(json_file, "w") as f:
            json.dump(flashcards, f, indent=4)
        print(f"Saved flashcards in JSON format to \"{json_file}\"")
    
    #! save the flashcards to anki-friendly text file
    save_to_text(flashcards, file_name=output_file, separator=anki_separator)
    print(f"Flashcards exported to anki-friendly text file to \"{output_file}\"")



import re


def get_flashcard(word, parser, log=False):
    # get definition of the word
    wiki_data = parser.fetch(word)
    definitions = []
    
    for w in wiki_data:
        definitions_set = []
        for d in w["definitions"]:
            definitions_set.extend(d["text"])
            
        definitions.append(definitions_set)

    if len(definitions) == 0 or len(definitions[0]) == 0:
        return None, None
    
    flashcard = []
    
    if log:
        print('\n==')
        print(word)
    for m in definitions:
        if log:
            print('--')
            print("\n".join(m))
            print()
        
        meaning = m[0]
        definitions = m[1:]
    
        flashcard.append({
            "meaning": meaning,
            "definitions": definitions
        })
    
    # check if the only definition is "* of *"
    first_definition = flashcard[0]['definitions'][0]
    
    base_re = re.compile(r".* of (\w*)\n")
    base_word = re.match(base_re, first_definition+"\n")

    if base_word:
        if log:
            print(f"base word: {base_word.group(1)}")
        
        return get_flashcard(base_word.group(1), parser)
        
    return word, flashcard



import sys
from PyQt6.QtCore import QObject, QThread, pyqtSignal
from PyQt6.QtWidgets import QApplication, QWidget, QLabel, QVBoxLayout, QPushButton, QFileDialog, QProgressBar, QLineEdit, QMessageBox

import clippings2anki.clippings as clippings


class Worker(QObject):
    finished = pyqtSignal()
    progress = pyqtSignal(int)
    
    def __init__(self, input_file, language, parent=None):
        super().__init__(parent)
        
        self.input_file = input_file
        self.language = language
        
        self.output_file = "Kindle flashcards.txt"
        
    def process(self):
        print("Processing...")
        
        clippings.main_cli(self.input_file, self.language, self.output_file, save_json=False, anki_separator="\t", qt_signal=self.progress)
            
        self.finished.emit()
        
    def finished_popup(self):
        print("Finished processing")
        
        popup = QMessageBox()
        popup.setWindowTitle("clippings2anki finished")
        popup.setText(f"Finished processing your Kindle clippings\nOutput file for anki import: {self.output_file}")
        _ = popup.exec()


class MainWindow(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Clippings2Anki")
        # self.resize(800, 600)
        # set minimum width
        self.setMinimumWidth(600)
        
        layout = QVBoxLayout()

        # add label
        title = QLabel("Clippings2Anki")
        layout.addWidget(title)

        # chosen file label
        self.chosen_file_label = QLabel("No file chosen")
        layout.addWidget(self.chosen_file_label)

        # choose file button
        self.input_file = None
        choose_file_button = QPushButton("Choose file")
        choose_file_button.clicked.connect(self.choose_file)
        layout.addWidget(choose_file_button)
        
        # language input
        language_label = QLabel("Language (e.g. \"english\", \"french\", \"german\")")
        self.language_input = QLineEdit("english")
        self.language_input.textChanged.connect(self.update_start_button)
        layout.addWidget(language_label)
        layout.addWidget(self.language_input)
        
        # start button
        self.start_button = QPushButton("Start")
        self.start_button.setEnabled(False)
        self.start_button.clicked.connect(self.start)
        layout.addWidget(self.start_button)
        
        # progress bar
        progress_bar_label = QLabel("Progress")
        layout.addWidget(progress_bar_label)
        
        self.progress_bar = QProgressBar()
        self.progress_bar.setMaximum(100)
        self.progress_bar.setValue(0)
        layout.addWidget(self.progress_bar)

        # show layout
        # layout.setContentsMargins(0,0,0,0)
        self.setLayout(layout)
        
    def choose_file(self):
        path = QFileDialog.getOpenFileName(self, 'Open a file', '', 'Kindle Clippings file (*.txt)')
        if path != ('', ''):
            print("File path : "+ path[0])
            
            # update label
            self.chosen_file_label.setText("Chosen file: " + path[0])
            self.input_file = path[0]
            
            self.update_start_button()
                
    def update_start_button(self):
        if self.language_input.text() != "" and self.input_file:
            self.start_button.setEnabled(True)
        else:
            self.start_button.setEnabled(False)
            
    def start(self):
        # disable start button
        self.start_button.setEnabled(False)
        
        self.progress_bar.setValue(0)
        
        # start processing
        self.worker_thread = QThread()
        self.worker = Worker(self.input_file, self.language_input.text())
        self.worker.progress.connect(self.progress_bar.setValue)    
        self.worker.moveToThread(self.worker_thread)
        self.worker.finished.connect(self.worker.finished_popup)
        self.worker.finished.connect(self.worker_thread.quit)
        self.worker.finished.connect(self.worker.deleteLater)
        self.worker_thread.started.connect(self.worker.process)
        self.worker_thread.finished.connect(self.worker_thread.deleteLater)
        self.worker_thread.start()
        

def main():
    app = QApplication(sys.argv)
    window = MainWindow()
    window.show()
    
    app.exec()
    

if __name__ == "__main__":
    main()



import sys
sys.path.append("")

import clippings2anki.clippings as clippings

if __name__ == "__main__":
    # if no arguments are given, show the gui
    if len(sys.argv) < 2:
        clippings.main_gui()
    
    else:
        # read cli arguments
        import argparse
        parser = argparse.ArgumentParser(description="Convert Kindle clippings to Anki flashcards")
        parser.add_argument("input_file", type=str, help="Path to the 'My clippings.txt' file")
        parser.add_argument("language", type=str, help="Language of the clippings (e.g. 'english' or 'german' for English or German)")
        parser.add_argument('-o', "--output", type=str, default="Kindle flashcards.txt", help="Path to the output file (default: 'Kindle flashcards.txt')")
        parser.add_argument('--json', action=argparse.BooleanOptionalAction, help="Save the parsed clippings as a json file (default: False)")
        parser.add_argument('--anki-separator', type=str, default="\t", help="Separator for the anki flashcard output (default: Tab)")
        args = parser.parse_args()

        clippings.main_cli(args.input_file, args.language, args.output, save_json=(args.json==True), anki_separator=args.anki_separator)
